/*
 * Copyright (c) 1993,1991,1990,1996  Arizona Board of Regents
 *
 * $RCSfile: scout_thread_prio.c,v $
 *
 * $Log: scout_thread_prio.c,v $
 * Revision 1.1  1997/01/28  22:50:41  rrp
 * Initial revision
 *
 * Revision 1.1  1996/05/28  23:19:50  davidm
 * Initial revision
 *
 */
/*
 * Fixed priority, round-robin scheduler.
 */
#include "xk_assert.h"
#include "platform.h"
#include "scout_queue.h"
#include "scout_thread.h"
#include "scout_thread_prio.h"

#define NPRIOS	(THREAD_PRIO_MIN - THREAD_PRIO_MAX + 1)

/*
 * Invariant: the currently running thread is "threadSelf" (it is
 * *not* in any ready_q!).
 */
static struct Queue	ready_q[NPRIOS];
static unsigned int	high_prio = THREAD_PRIO_MAX;


static void
prio_wakeup_thread (Thread t)
{
    xAssert((unsigned int) (t->option.priority - THREAD_PRIO_MAX) 
             <= THREAD_PRIO_MIN);
    queueAppend(&ready_q[t->option.priority - THREAD_PRIO_MAX], t);
    if (t->option.priority < high_prio) {
	high_prio = t->option.priority;
    }
}


/*
 * Return highest priority thread or 0 if there is none.
 */
static Thread
prio_select_thread (void)
{
    Thread next;

    for (; high_prio <= THREAD_PRIO_MIN; ++high_prio) {
	next = queueRemove(&ready_q[high_prio - THREAD_PRIO_MAX]);
	if (next) {
	    return next;
	}
    }
    return 0;
}


void
threadInitFixedPrioScheduler (void)
{
    unsigned int prio;

    for (prio = THREAD_PRIO_MAX; prio <= THREAD_PRIO_MIN; ++prio) {
	queueInit(&ready_q[prio - THREAD_PRIO_MAX]);
    }
    threadRegisterScheduler(&threadFixedPrioScheduler);
}


struct ThreadScheduler threadFixedPrioScheduler = {
    "Fixed-priority round-robin scheduler",
    prio_wakeup_thread,
    prio_select_thread,
    1						/* quota */
};
